body{margin:0;padding:0;font-family:-apple-system,BlinkMacSystemFont,Segoe UI,Roboto,Oxygen,Ubuntu,Cantarell,Fira Sans,Droid Sans,Helvetica Neue,sans-serif;-webkit-font-smoothing:antialiased;-moz-osx-font-smoothing:grayscale}code{font-family:source-code-pro,Menlo,Monaco,Consolas,Courier New,monospace}html{font-family:arial}a,body{color:#fff}#terminal{margin:40px;padding:10px;min-height:100px;background-color:#000}#terminal.connection-pending{box-shadow:0 0 20px 8px #9bbaff}#terminal.connected{box-shadow:0 0 20px 8px #a1b570}#terminal.disconnected{box-shadow:0 0 20px 8px #bf3333}td{padding:3px}body{padding:50px;background-color:#222}.led{width:10px;height:10px;border-radius:50%;background-color:hsla(0,0%,100%,.25);box-shadow:0 -1px 6px 1px #000}.led-red{background-color:red;box-shadow:0 -1px 6px 1px #000,inset 0 -1px 8px #600,0 3px 11px red}.led-orange{background-color:#ff7000;box-shadow:0 -1px 6px 1px #000,inset 0 -1px 8px #630,0 3px 11px #ff7000}.led-yellow{background-color:#ff0;box-shadow:0 -1px 6px 1px #000,inset 0 -1px 8px #660,0 3px 11px #ff0}.led-green{background-color:#80ff00;box-shadow:0 -1px 6px 1px #000,inset 0 -1px 8px #460,0 3px 11px #80ff00}.led-blue{background-color:#06f;box-shadow:0 -1px 6px 1px #000,inset 0 -1px 8px #006,0 3px 11px #06f}
/*# sourceMappingURL=main.6d80e0d4.chunk.css.map */