self.__precacheManifest = [
  {
    "revision": "74cdedb8050c1198d3c477e62d8bd849",
    "url": "/static/media/logo.74cdedb8.png"
  },
  {
    "revision": "fdfcfda2d9b1bf31db52",
    "url": "/static/js/runtime~main.c5541365.js"
  },
  {
    "revision": "783dcf157b3b7d4f7b75",
    "url": "/static/js/main.418b1700.chunk.js"
  },
  {
    "revision": "d5a28764b5f36482a53f",
    "url": "/static/js/2.c31ec014.chunk.js"
  },
  {
    "revision": "783dcf157b3b7d4f7b75",
    "url": "/static/css/main.6d80e0d4.chunk.css"
  },
  {
    "revision": "715f6fcc24ea5871ff3cde20fe0a5440",
    "url": "/index.html"
  }
];